/*
 * File:         roombots.c
 * Date:         December 20th, 2006
 * Description:  The custom ODE pysics plugin used to speed up the Roombots
 *               simulations.
 * Author:       Simon Blanchoud
 * Modifications:
 */

#include <physics/physics.h>

/* The general variables from ODE. */
static dWorldID world;
static dSpaceID space;
static dJointGroupID contact_joint_group;

/* The two parts which still have collisions together. */
static dGeomID ground;
static dGeomID groundbot;
/* The only part which still have collisions with the modules. */
static dGeomID base;

/*
 * The initialization function, here we store the general variables and we 
 * retrieve the geoms.
 */
void webots_physics_init(dWorldID w, dSpaceID s, dJointGroupID j)
{
    world = w;
    space = s;
    contact_joint_group = j;

    /* We retrieve the geom from our simulations. */
    ground = dWebotsGetGeomFromDEF("GROUND");
    groundbot = dWebotsGetGeomFromDEF("GROUNDBOT");
    base = dWebotsGetGeomFromDEF("BASE");

    return;
}

/*
 * This function is called every time a collision is detected. We use this
 * function to avoid useless colision between te parts of the robot itself.
 */
int webots_physics_collide(dGeomID g1, dGeomID g2)
{
  /*
   * If the base element exists, then it is the only solid which has
   * collisions with everything else.
   */
  if (base != NULL) {
    if (base == g1 || base == g2) {
      return 0;
    }
  }

  /*
   * If the collision is between the ground and the groundbot we let ODE handle
   * the collisions so that the groundbot can stay on the ground. If not, we skip it.
   */
  if (dGeomIsSpace(groundbot)) {
    if ((dSpaceQuery((dSpaceID) groundbot, g1) == 1 && ground == g2) || (dSpaceQuery((dSpaceID) groundbot, g2) == 1 && ground == g1)) {
      return 0;
    } else {
      return 1;
    }
  } else {
    if ((groundbot == g1 && ground == g2) || (groundbot == g2 && ground == g1)) {
      return 0;
    } else {
      return 1;
    }
  }

  return 0;
}
